/*
 * #%L
 * JavaHg
 * %%
 * Copyright (C) 2011 aragost Trifork ag
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
package com.aragost.javahg;

import java.io.File;
import java.nio.charset.CharsetDecoder;
import java.util.List;

import com.aragost.javahg.internals.ServerPool;

/**
 * This class represents a repository with an overlaid bundle
 */
class OverlayRepository extends Repository {

    /**
     * If not null this is the overlay bundle for the repository.
     */
    private Bundle bundle;

    private BaseRepository baseRepository;

    public OverlayRepository(BaseRepository repository, Bundle bundle) {
        // Note that since the bundle has strong ref to changesets
        // from the bundle even with weak or soft cache policy they
        // will never be evicted
        super(repository.getConfiguration().getCachePolicy());
        this.baseRepository = repository;
        this.bundle = bundle;
        repository.getServerPool().incrementRefCount();
    }

    public Bundle getBundle() {
        return this.bundle;
    }

    @Override
    public File getDirectory() {
        return getBaseRepository().getDirectory();
    }

    public BaseRepository getBaseRepository() {
        return baseRepository;
    }

    @Override
    public ServerPool getServerPool() {
        return getBaseRepository().getServerPool();
    }

    @Override
    public void addToCommandLine(List<String> commandLine) {
        super.addToCommandLine(commandLine);
        commandLine.add("--repository");
        commandLine.add(getBundle().getFile().getPath());
    }

    @Override
    public CharsetDecoder newDecoder() {
        return baseRepository.newDecoder();
    }
}
