/*
 * #%L
 * JavaHg
 * %%
 * Copyright (C) 2011 aragost Trifork ag
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
package com.aragost.javahg.commands;

import java.util.List;

import com.aragost.javahg.Repository;
import com.aragost.javahg.commands.flags.BranchesCommandFlags;
import com.google.common.collect.Lists;

/**
 * Command class for executing <tt>hg branches</tt>. Set flags from
 * {@link BranchesCommandFlags} and see the {@link #execute()} method
 * for how to run the command.
 * <p>
 * This class wont be able to parse the branch names correctly if they
 * contain newline characters (Mercurial will unfortunately let you
 * create such branch names).
 */
public class BranchesCommand extends BranchesCommandFlags {

    public BranchesCommand(Repository repository) {
        super(repository);
        withDebugFlag();
    }

    /**
     * @return list of branches for the repository
     */
    public List<Branch> execute() {
        List<Branch> branches = Lists.newArrayList();
        for (String line : launchIterator()) {
            Branch branch = Branch.fromLine(getRepository(), line);
            branches.add(branch);
        }
        return branches;
    }

}
