/*
 * #%L
 * JavaHg
 * %%
 * Copyright (C) 2011 aragost Trifork ag
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
package com.aragost.javahg.commands;

import java.io.IOException;
import java.util.List;

import com.aragost.javahg.Changeset;
import com.aragost.javahg.Repository;
import com.aragost.javahg.commands.flags.PullCommandFlags;
import com.aragost.javahg.internals.PullPushHelper;

/**
 * Command class for executing <tt>hg pull</tt>. Set flags from
 * {@link PullCommandFlags} and call the {@link #execute} method.
 */
public class PullCommand extends PullCommandFlags {

    /**
     * @param repository
     *            the repository associated with this command.
     */
    public PullCommand(Repository repository) {
        super(repository);
        withDebugFlag();
    }

    @Override
    public boolean isSuccessful() {
        return super.isSuccessful() || getReturnCode() == 1;
    }

    private List<Changeset> run(String source) throws IOException {
        if (source != null) {
            cmdAppend(source);
        }
        // TODO: handle pushing multiple heads
        return PullPushHelper.parseStream(getRepository(), launchStream());
    }

    /**
     * Run <tt>hg pull</tt> using the default path.
     * 
     * @return list of pulled changesets.
     * @throws IOException
     */
    public List<Changeset> execute() throws IOException {
        return run(null);
    }

    /**
     * Run <tt>hg pull source</tt> with an explicit source path.
     * 
     * @param source
     *            the source path. Cannot be null.
     * 
     * @return list of pulled changesets.
     * @throws IOException
     */
    public List<Changeset> execute(String source) throws IOException {
        if (source == null) {
            throw new NullPointerException("source cannot be null");
        }
        return run(source);
    }

}
